@@ -1,10 +1,8 @@
 #ifndef MC__FILEHIGHLIGHT_H
 #define MC__FILEHIGHLIGHT_H
 
-#include "lib/mcconfig.h"
-#include "lib/search.h"
-
-#include "lib/util.h"
+#include "lib/mcconfig.h"       /* mc_config_t */
+#include "lib/util.h"           /* file_entry_t */
 
 /*** typedefs(not structures) and defined constants **********************************************/
 
