@@ -1,7 +1,7 @@
 /*
    paths to configuration files
 
-   Copyright (C) 2010-2019
+   Copyright (C) 2010-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -63,37 +63,35 @@ static const struct
 {
     /* *INDENT-OFF* */
     /* config */
-    { "ini",                                   &mc_config_str, MC_CONFIG_FILE},
-    { "filehighlight.ini",                     &mc_config_str, MC_FHL_INI_FILE},
-    { "hotlist",                               &mc_config_str, MC_HOTLIST_FILE},
-    { "mc.keymap",                             &mc_config_str, GLOBAL_KEYMAP_FILE},
-    { "menu",                                  &mc_config_str, MC_USERMENU_FILE},
-    { "cedit" PATH_SEP_STR "Syntax",           &mc_config_str, EDIT_SYNTAX_FILE},
-    { "cedit" PATH_SEP_STR "menu",             &mc_config_str, EDIT_HOME_MENU},
-    { "cedit" PATH_SEP_STR "edit.indent.rc",   &mc_config_str, EDIT_DIR PATH_SEP_STR "edit.indent.rc"},
-    { "cedit" PATH_SEP_STR "edit.spell.rc",    &mc_config_str, EDIT_DIR PATH_SEP_STR "edit.spell.rc"},
-    { "panels.ini",                            &mc_config_str, MC_PANELS_FILE},
+    { "ini",                                 &mc_config_str, MC_CONFIG_FILE },
+    { "filehighlight.ini",                   &mc_config_str, MC_FHL_INI_FILE },
+    { "hotlist",                             &mc_config_str, MC_HOTLIST_FILE },
+    { "mc.keymap",                           &mc_config_str, GLOBAL_KEYMAP_FILE },
+    { "menu",                                &mc_config_str, MC_USERMENU_FILE },
+    { "cedit" PATH_SEP_STR "Syntax",         &mc_config_str, EDIT_HOME_SYNTAX_FILE },
+    { "cedit" PATH_SEP_STR "menu",           &mc_config_str, EDIT_HOME_MENU },
+    { "panels.ini",                          &mc_config_str, MC_PANELS_FILE },
 
     /* User should move this file with applying some changes in file */
-    { "",                                      &mc_config_str, MC_FILEBIND_FILE},
+    { "",                                    &mc_config_str, MC_FILEBIND_FILE },
 
     /* data */
-    { "skins",                                 &mc_data_str, MC_SKINS_SUBDIR},
-    { "fish",                                  &mc_data_str, FISH_PREFIX},
-    { "ashrc",                                 &mc_data_str, "ashrc"},
-    { "bashrc",                                &mc_data_str, "bashrc"},
-    { "inputrc",                               &mc_data_str, "inputrc"},
-    { "extfs.d",                               &mc_data_str, MC_EXTFS_DIR},
-    { "history",                               &mc_data_str, MC_HISTORY_FILE},
-    { "filepos",                               &mc_data_str, MC_FILEPOS_FILE},
-    { "cedit" PATH_SEP_STR "cooledit.clip",    &mc_data_str, EDIT_CLIP_FILE},
-    { "",                                      &mc_data_str, MC_MACRO_FILE},
+    { "skins",                               &mc_data_str, MC_SKINS_DIR },
+    { "fish",                                &mc_data_str, FISH_PREFIX },
+    { "ashrc",                               &mc_data_str, "ashrc" },
+    { "bashrc",                              &mc_data_str, "bashrc" },
+    { "inputrc",                             &mc_data_str, "inputrc" },
+    { "extfs.d",                             &mc_data_str, MC_EXTFS_DIR },
+    { "history",                             &mc_data_str, MC_HISTORY_FILE },
+    { "filepos",                             &mc_data_str, MC_FILEPOS_FILE },
+    { "cedit" PATH_SEP_STR "cooledit.clip",  &mc_data_str, EDIT_HOME_CLIP_FILE },
+    { "",                                    &mc_data_str, MC_MACRO_FILE },
 
     /* cache */
-    { "log",                                   &mc_cache_str, "mc.log"},
-    { "Tree",                                  &mc_cache_str, MC_TREESTORE_FILE},
-    { "cedit" PATH_SEP_STR "cooledit.temp",    &mc_cache_str, EDIT_TEMP_FILE},
-    { "cedit" PATH_SEP_STR "cooledit.block",   &mc_cache_str, EDIT_BLOCK_FILE},
+    { "log",                                 &mc_cache_str, "mc.log" },
+    { "Tree",                                &mc_cache_str, MC_TREESTORE_FILE },
+    { "cedit" PATH_SEP_STR "cooledit.temp",  &mc_cache_str, EDIT_HOME_TEMP_FILE },
+    { "cedit" PATH_SEP_STR "cooledit.block", &mc_cache_str, EDIT_HOME_BLOCK_FILE },
 
     {NULL, NULL, NULL}
     /* *INDENT-ON* */
@@ -109,20 +107,17 @@ static const struct
 } mc_config_migrate_rules_fix[] =
 {
     /* *INDENT-OFF* */
-    { &mc_data_str, MC_USERMENU_FILE,                       &mc_config_str},
-    { &mc_data_str, EDIT_SYNTAX_FILE,                       &mc_config_str},
-    { &mc_data_str, EDIT_HOME_MENU,                         &mc_config_str},
-    { &mc_data_str, EDIT_DIR PATH_SEP_STR "edit.indent.rc", &mc_config_str},
-    { &mc_data_str, EDIT_DIR PATH_SEP_STR "edit.spell.rc",  &mc_config_str},
-    { &mc_data_str, MC_FILEBIND_FILE,                       &mc_config_str},
+    { &mc_data_str, MC_USERMENU_FILE,      &mc_config_str },
+    { &mc_data_str, MC_FILEBIND_FILE,      &mc_config_str },
+    { &mc_data_str, EDIT_HOME_SYNTAX_FILE, &mc_config_str },
+    { &mc_data_str, EDIT_HOME_MENU,        &mc_config_str },
 
-    { &mc_cache_str, MC_HISTORY_FILE,                       &mc_data_str},
-    { &mc_cache_str, MC_FILEPOS_FILE,                       &mc_data_str},
-    { &mc_cache_str, EDIT_CLIP_FILE,                        &mc_data_str},
+    { &mc_cache_str, MC_PANELS_FILE,       &mc_config_str },
+    { &mc_cache_str, MC_HISTORY_FILE,      &mc_data_str },
+    { &mc_cache_str, MC_FILEPOS_FILE,      &mc_data_str },
+    { &mc_cache_str, EDIT_HOME_CLIP_FILE,  &mc_data_str },
 
-    { &mc_cache_str, MC_PANELS_FILE,                        &mc_config_str},
-
-    {NULL, NULL, NULL}
+    { NULL, NULL, NULL }
     /* *INDENT-ON* */
 };
 #endif /* MC_HOMEDIR_XDG */
@@ -435,10 +430,10 @@ mc_config_migrate_from_old_place (GError ** mcerror, char **msg)
 
     old_dir = mc_config_get_deprecated_path ();
 
-    g_free (mc_config_init_one_config_path (mc_config_str, EDIT_DIR, mcerror));
+    g_free (mc_config_init_one_config_path (mc_config_str, EDIT_HOME_DIR, mcerror));
 #if MC_HOMEDIR_XDG
-    g_free (mc_config_init_one_config_path (mc_cache_str, EDIT_DIR, mcerror));
-    g_free (mc_config_init_one_config_path (mc_data_str, EDIT_DIR, mcerror));
+    g_free (mc_config_init_one_config_path (mc_cache_str, EDIT_HOME_DIR, mcerror));
+    g_free (mc_config_init_one_config_path (mc_data_str, EDIT_HOME_DIR, mcerror));
 #endif /* MC_HOMEDIR_XDG */
 
     mc_return_val_if_error (mcerror, FALSE);
