@@ -56,6 +56,7 @@
 
 #include "tty.h"
 #include "tty-internal.h"
+#include "color.h"              /* tty_set_normal_attrs() */
 #include "mouse.h"              /* use_mouse_p */
 #include "win.h"
 
@@ -264,6 +265,17 @@ tty_draw_box (int y, int x, int ys, int xs, gboolean single)
 
 /* --------------------------------------------------------------------------------------------- */
 
+void
+tty_draw_box_shadow (int y, int x, int rows, int cols, int shadow_color)
+{
+    /* draw right shadow */
+    tty_colorize_area (y + 1, x + cols, rows - 1, 2, shadow_color);
+    /* draw bottom shadow */
+    tty_colorize_area (y + rows, x + 2, 1, cols, shadow_color);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 char *
 mc_tty_normalize_from_utf8 (const char *str)
 {
@@ -312,6 +324,17 @@ tty_resize (int fd)
 
 /* --------------------------------------------------------------------------------------------- */
 
+/** Clear screen */
+void
+tty_clear_screen (void)
+{
+    tty_set_normal_attrs ();
+    tty_fill_region (0, 0, LINES, COLS, ' ');
+    tty_refresh ();
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 void
 tty_init_xterm_support (gboolean is_xterm)
 {
