@@ -28,8 +28,6 @@ int dialog_switch_process_pending (void);
 void dialog_switch_got_winch (void);
 void dialog_switch_shutdown (void);
 
-/* Clear screen */
-void clr_scr (void);
 void repaint_screen (void);
 void mc_refresh (void);
 void dialog_change_screen_size (void);
