@@ -13,7 +13,7 @@ do_view_action() {
 
     case "${filetype}" in
     *)
-        if mplayer >/dev/null 2>&1; then
+        if which mplayer >/dev/null 2>&1; then
             mplayer -identify -vo null -ao null -frames 0 "${MC_EXT_FILENAME}" 2>&1 | \
                 sed -n 's/^ID_//p'
         elif which mpv_identify.sh >/dev/null 2>&1; then
@@ -28,9 +28,9 @@ do_view_action() {
 do_open_action() {
     filetype=$1
 
-    if mpv >/dev/null 2>&1; then
+    if which mpv >/dev/null 2>&1; then
         PLAYER=mpv
-    elif mplayer >/dev/null 2>&1; then
+    elif which mplayer >/dev/null 2>&1; then
         PLAYER=mplayer
     else
         echo "Please install either mplayer or mpv to play this file"
