@@ -1,7 +1,7 @@
 /*
    Editor initialisation and callback handler.
 
-   Copyright (C) 1996-2019
+   Copyright (C) 1996-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -45,7 +45,7 @@
 #include "lib/tty/key.h"        /* is_idle() */
 #include "lib/tty/color.h"      /* tty_setcolor() */
 #include "lib/skin.h"
-#include "lib/fileloc.h"        /* EDIT_DIR */
+#include "lib/fileloc.h"        /* EDIT_HOME_DIR */
 #include "lib/strutil.h"        /* str_term_trim() */
 #include "lib/util.h"           /* mc_build_filename() */
 #include "lib/widget.h"
@@ -146,7 +146,7 @@ edit_about (void)
         QUICK_LABEL (N_("A user friendly text editor\n"
                         "written for the Midnight Commander."), NULL),
         QUICK_SEPARATOR (FALSE),
-        QUICK_LABEL (N_("Copyright (C) 1996-2019 the Free Software Foundation"), NULL),
+        QUICK_LABEL (N_("Copyright (C) 1996-2020 the Free Software Foundation"), NULL),
         QUICK_START_BUTTONS (TRUE, TRUE),
             QUICK_BUTTON (N_("&OK"), B_ENTER, NULL, NULL),
         QUICK_END
@@ -192,8 +192,8 @@ edit_restore_size (WEdit * edit)
 
     edit->drag_state = MCEDIT_DRAG_NONE;
     w->mouse.forced_capture = FALSE;
-    widget_set_size (w, edit->y_prev, edit->x_prev, edit->lines_prev, edit->cols_prev);
-    dlg_draw (w->owner);
+    widget_set_size_rect (w, &edit->loc_prev);
+    widget_draw (WIDGET (w->owner));
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -233,7 +233,7 @@ edit_window_move (WEdit * edit, long command)
     }
 
     edit->force |= REDRAW_PAGE;
-    dlg_draw (w->owner);
+    widget_draw (WIDGET (w->owner));
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -273,7 +273,7 @@ edit_window_resize (WEdit * edit, long command)
     }
 
     edit->force |= REDRAW_COMPLETELY;
-    dlg_draw (w->owner);
+    widget_draw (WIDGET (w->owner));
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -295,7 +295,9 @@ get_hotkey (int n)
 static void
 edit_window_list (const WDialog * h)
 {
-    const size_t dlg_num = g_list_length (h->widgets) - 2;      /* 2 = skip menu and buttonbar */
+    const WGroup *g = CONST_GROUP (h);
+    const size_t offset = 2;    /* skip menu and buttonbar */
+    const size_t dlg_num = g_list_length (g->widgets) - offset;
     int lines, cols;
     Listbox *listbox;
     GList *w;
@@ -307,7 +309,7 @@ edit_window_list (const WDialog * h)
 
     listbox = create_listbox_window (lines, cols, _("Open files"), "[Open files]");
 
-    for (w = h->widgets; w != NULL; w = g_list_next (w))
+    for (w = g->widgets; w != NULL; w = g_list_next (w))
         if (edit_widget_is_editor (CONST_WIDGET (w->data)))
         {
             WEdit *e = (WEdit *) w->data;
@@ -325,7 +327,7 @@ edit_window_list (const WDialog * h)
             g_free (fname);
         }
 
-    selected = run_listbox_with_data (listbox, h->current->data);
+    selected = run_listbox_with_data (listbox, g->current->data);
     if (selected != NULL)
         widget_select (WIDGET (selected));
 }
@@ -379,6 +381,7 @@ edit_get_title (const WDialog * h, size_t len)
 static cb_ret_t
 edit_dialog_command_execute (WDialog * h, long command)
 {
+    WGroup *g = GROUP (h);
     Widget *wh = WIDGET (h);
     cb_ret_t ret = MSG_HANDLED;
 
@@ -401,8 +404,8 @@ edit_dialog_command_execute (WDialog * h, long command)
         break;
     case CK_Close:
         /* if there are no opened files anymore, close MC editor */
-        if (edit_widget_is_editor (CONST_WIDGET (h->current->data)) &&
-            edit_close_cmd ((WEdit *) h->current->data) && find_editor (h) == NULL)
+        if (edit_widget_is_editor (CONST_WIDGET (g->current->data)) &&
+            edit_close_cmd ((WEdit *) g->current->data) && find_editor (h) == NULL)
             dlg_stop (h);
         break;
     case CK_Help:
@@ -416,7 +419,7 @@ edit_dialog_command_execute (WDialog * h, long command)
     case CK_Cancel:
         /* don't close editor due to SIGINT, but stop move/resize window */
         {
-            Widget *w = WIDGET (h->current->data);
+            Widget *w = WIDGET (g->current->data);
 
             if (edit_widget_is_editor (w) && ((WEdit *) w)->drag_state != MCEDIT_DRAG_NONE)
                 edit_restore_size ((WEdit *) w);
@@ -450,17 +453,17 @@ edit_dialog_command_execute (WDialog * h, long command)
         break;
     case CK_WindowMove:
     case CK_WindowResize:
-        if (edit_widget_is_editor (CONST_WIDGET (h->current->data)))
-            edit_handle_move_resize ((WEdit *) h->current->data, command);
+        if (edit_widget_is_editor (CONST_WIDGET (g->current->data)))
+            edit_handle_move_resize ((WEdit *) g->current->data, command);
         break;
     case CK_WindowList:
         edit_window_list (h);
         break;
     case CK_WindowNext:
-        dlg_select_next_widget (h);
+        group_select_next_widget (g);
         break;
     case CK_WindowPrev:
-        dlg_select_prev_widget (h);
+        group_select_prev_widget (g);
         break;
     case CK_Options:
         edit_options_dialog (h);
@@ -488,11 +491,12 @@ edit_dialog_command_execute (WDialog * h, long command)
 static gboolean
 edit_translate_key (WEdit * edit, long x_key, int *cmd, int *ch)
 {
+    Widget *w = WIDGET (edit);
     long command = CK_InsertChar;
     int char_for_insertion = -1;
 
     /* an ordinary insertable character */
-    if (!edit->extmod && x_key < 256)
+    if (!w->ext_mode && x_key < 256)
     {
 #ifndef HAVE_CHARSET
         if (is_printable (x_key))
@@ -585,14 +589,7 @@ edit_translate_key (WEdit * edit, long x_key, int *cmd, int *ch)
     }
 
     /* Commands specific to the key emulation */
-    if (edit->extmod)
-    {
-        edit->extmod = FALSE;
-        command = keybind_lookup_keymap_command (editor_x_map, x_key);
-    }
-    else
-        command = keybind_lookup_keymap_command (editor_map, x_key);
-
+    command = widget_lookup_key (w, x_key);
     if (command == CK_IgnoreKey)
         command = CK_InsertChar;
 
@@ -618,7 +615,7 @@ edit_quit (WDialog * h)
     widget_set_state (WIDGET (h), WST_ACTIVE, TRUE);
 
     /* check window state and get modified files */
-    for (l = h->widgets; l != NULL; l = g_list_next (l))
+    for (l = GROUP (h)->widgets; l != NULL; l = g_list_next (l))
         if (edit_widget_is_editor (CONST_WIDGET (l->data)))
         {
             e = (WEdit *) l->data;
@@ -658,16 +655,18 @@ edit_quit (WDialog * h)
 static inline void
 edit_set_buttonbar (WEdit * edit, WButtonBar * bb)
 {
-    buttonbar_set_label (bb, 1, Q_ ("ButtonBar|Help"), editor_map, NULL);
-    buttonbar_set_label (bb, 2, Q_ ("ButtonBar|Save"), editor_map, WIDGET (edit));
-    buttonbar_set_label (bb, 3, Q_ ("ButtonBar|Mark"), editor_map, WIDGET (edit));
-    buttonbar_set_label (bb, 4, Q_ ("ButtonBar|Replac"), editor_map, WIDGET (edit));
-    buttonbar_set_label (bb, 5, Q_ ("ButtonBar|Copy"), editor_map, WIDGET (edit));
-    buttonbar_set_label (bb, 6, Q_ ("ButtonBar|Move"), editor_map, WIDGET (edit));
-    buttonbar_set_label (bb, 7, Q_ ("ButtonBar|Search"), editor_map, WIDGET (edit));
-    buttonbar_set_label (bb, 8, Q_ ("ButtonBar|Delete"), editor_map, WIDGET (edit));
-    buttonbar_set_label (bb, 9, Q_ ("ButtonBar|PullDn"), editor_map, NULL);
-    buttonbar_set_label (bb, 10, Q_ ("ButtonBar|Quit"), editor_map, NULL);
+    Widget *w = WIDGET (edit);
+
+    buttonbar_set_label (bb, 1, Q_ ("ButtonBar|Help"), w->keymap, NULL);
+    buttonbar_set_label (bb, 2, Q_ ("ButtonBar|Save"), w->keymap, w);
+    buttonbar_set_label (bb, 3, Q_ ("ButtonBar|Mark"), w->keymap, w);
+    buttonbar_set_label (bb, 4, Q_ ("ButtonBar|Replac"), w->keymap, w);
+    buttonbar_set_label (bb, 5, Q_ ("ButtonBar|Copy"), w->keymap, w);
+    buttonbar_set_label (bb, 6, Q_ ("ButtonBar|Move"), w->keymap, w);
+    buttonbar_set_label (bb, 7, Q_ ("ButtonBar|Search"), w->keymap, w);
+    buttonbar_set_label (bb, 8, Q_ ("ButtonBar|Delete"), w->keymap, w);
+    buttonbar_set_label (bb, 9, Q_ ("ButtonBar|PullDn"), w->keymap, NULL);
+    buttonbar_set_label (bb, 10, Q_ ("ButtonBar|Quit"), w->keymap, NULL);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -746,6 +745,7 @@ edit_update_cursor (WEdit * edit, const mouse_event_t * event)
 static cb_ret_t
 edit_dialog_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
+    WGroup *g = GROUP (w);
     WDialog *h = DIALOG (w);
 
     switch (msg)
@@ -754,12 +754,6 @@ edit_dialog_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, v
         edit_dlg_init ();
         return MSG_HANDLED;
 
-    case MSG_DRAW:
-        /* don't use dlg_default_repaint() -- we don't need a frame */
-        tty_setcolor (EDITOR_BACKGROUND);
-        dlg_erase (h);
-        return MSG_HANDLED;
-
     case MSG_RESIZE:
         dlg_default_callback (w, NULL, MSG_RESIZE, 0, NULL);
         menubar_arrange (find_menubar (h));
@@ -776,35 +770,35 @@ edit_dialog_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, v
             /* We forward any commands coming from the menu, and which haven't been
                handled by the dialog, to the focused WEdit window. */
             if (result == MSG_NOT_HANDLED && sender == WIDGET (find_menubar (h)))
-                result = send_message (h->current->data, NULL, MSG_ACTION, parm, NULL);
+                result = send_message (g->current->data, NULL, MSG_ACTION, parm, NULL);
 
             return result;
         }
 
     case MSG_KEY:
         {
-            Widget *we = WIDGET (h->current->data);
+            Widget *we = WIDGET (g->current->data);
             cb_ret_t ret = MSG_NOT_HANDLED;
 
             if (edit_widget_is_editor (we))
             {
-                WEdit *e = (WEdit *) we;
+                gboolean ext_mode;
                 long command;
 
-                if (!e->extmod)
-                    command = keybind_lookup_keymap_command (editor_map, parm);
-                else
-                    command = keybind_lookup_keymap_command (editor_x_map, parm);
+                /* keep and then extmod flag */
+                ext_mode = we->ext_mode;
+                command = widget_lookup_key (we, parm);
+                we->ext_mode = ext_mode;
 
                 if (command == CK_IgnoreKey)
-                    e->extmod = FALSE;
+                    we->ext_mode = FALSE;
                 else
                 {
                     ret = edit_dialog_command_execute (h, command);
                     /* if command was not handled, keep the extended mode
                        for the further key processing */
                     if (ret == MSG_HANDLED)
-                        e->extmod = FALSE;
+                        we->ext_mode = FALSE;
                 }
             }
 
@@ -834,7 +828,7 @@ edit_dialog_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, v
 
     case MSG_IDLE:
         widget_idle (w, FALSE);
-        return send_message (h->current->data, NULL, MSG_IDLE, 0, NULL);
+        return send_message (g->current->data, NULL, MSG_IDLE, 0, NULL);
 
     default:
         return dlg_default_callback (w, sender, msg, parm, data);
@@ -857,6 +851,7 @@ edit_dialog_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
 
     if (msg == MSG_MOUSE_DOWN && event->y == 0)
     {
+        WGroup *g = GROUP (w);
         WDialog *h = DIALOG (w);
         WMenuBar *b;
 
@@ -871,7 +866,7 @@ edit_dialog_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
             int x;
 
             /* Try find top fullscreen window */
-            for (l = h->widgets; l != NULL; l = g_list_next (l))
+            for (l = g->widgets; l != NULL; l = g_list_next (l))
                 if (edit_widget_is_editor (CONST_WIDGET (l->data))
                     && ((WEdit *) l->data)->fullscreen)
                     top = l;
@@ -883,7 +878,7 @@ edit_dialog_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
             {
                 WEdit *e = (WEdit *) top->data;
 
-                if (top != h->current)
+                if (top != g->current)
                 {
                     /* Window is not active. Activate it */
                     widget_select (WIDGET (e));
@@ -909,6 +904,31 @@ edit_dialog_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
 
 /* --------------------------------------------------------------------------------------------- */
 
+static cb_ret_t
+edit_dialog_bg_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
+{
+    switch (msg)
+    {
+    case MSG_INIT:
+        {
+            Widget *wo = WIDGET (w->owner);
+
+            w->y = wo->y + 1;
+            w->x = wo->x;
+            w->lines = wo->lines - 2;
+            w->cols = wo->cols;
+            w->pos_flags |= WPOS_KEEP_ALL;
+
+            return MSG_HANDLED;
+        }
+
+    default:
+        return background_callback (w, sender, msg, parm, data);
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 static cb_ret_t
 edit_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
@@ -917,7 +937,7 @@ edit_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *da
     switch (msg)
     {
     case MSG_FOCUS:
-        edit_set_buttonbar (e, find_buttonbar (w->owner));
+        edit_set_buttonbar (e, find_buttonbar (DIALOG (w->owner)));
         return MSG_HANDLED;
 
     case MSG_DRAW:
@@ -1029,7 +1049,7 @@ edit_mouse_handle_move_resize (Widget * w, mouse_msg_t msg, mouse_event_t * even
     edit->force |= REDRAW_COMPLETELY;   /* Not really needed as WEdit's MSG_DRAW already does this. */
 
     /* We draw the whole dialog because dragging/resizing exposes area beneath. */
-    dlg_draw (w->owner);
+    widget_draw (WIDGET (w->owner));
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1187,6 +1207,7 @@ edit_files (const GList * files)
 {
     static gboolean made_directory = FALSE;
     WDialog *edit_dlg;
+    WGroup *g;
     WMenuBar *menubar;
     Widget *w, *wd;
     const GList *file;
@@ -1196,15 +1217,15 @@ edit_files (const GList * files)
     {
         char *dir;
 
-        dir = mc_build_filename (mc_config_get_cache_path (), EDIT_DIR, (char *) NULL);
+        dir = mc_build_filename (mc_config_get_cache_path (), EDIT_HOME_DIR, (char *) NULL);
         made_directory = (mkdir (dir, 0700) != -1 || errno == EEXIST);
         g_free (dir);
 
-        dir = mc_build_filename (mc_config_get_path (), EDIT_DIR, (char *) NULL);
+        dir = mc_build_filename (mc_config_get_path (), EDIT_HOME_DIR, (char *) NULL);
         made_directory = (mkdir (dir, 0700) != -1 || errno == EEXIST);
         g_free (dir);
 
-        dir = mc_build_filename (mc_config_get_data_path (), EDIT_DIR, (char *) NULL);
+        dir = mc_build_filename (mc_config_get_data_path (), EDIT_HOME_DIR, (char *) NULL);
         made_directory = (mkdir (dir, 0700) != -1 || errno == EEXIST);
         g_free (dir);
     }
@@ -1215,17 +1236,26 @@ edit_files (const GList * files)
                     edit_dialog_mouse_callback, "[Internal File Editor]", NULL);
     wd = WIDGET (edit_dlg);
     widget_want_tab (wd, TRUE);
+    wd->keymap = editor_map;
+    wd->ext_keymap = editor_x_map;
 
     edit_dlg->get_shortcut = edit_get_shortcut;
     edit_dlg->get_title = edit_get_title;
 
+    g = GROUP (edit_dlg);
+
+    edit_dlg->bg =
+        WIDGET (background_new
+                (1, 0, wd->lines - 2, wd->cols, EDITOR_BACKGROUND, ' ', edit_dialog_bg_callback));
+    group_add_widget (g, edit_dlg->bg);
+
     menubar = menubar_new (NULL, TRUE);
     w = WIDGET (menubar);
-    add_widget_autopos (edit_dlg, w, w->pos_flags, NULL);
+    group_add_widget_autopos (g, w, w->pos_flags, NULL);
     edit_init_menu (menubar);
 
     w = WIDGET (buttonbar_new (TRUE));
-    add_widget_autopos (edit_dlg, w, w->pos_flags, NULL);
+    group_add_widget_autopos (g, w, w->pos_flags, NULL);
 
     for (file = files; file != NULL; file = g_list_next (file))
     {
@@ -1260,9 +1290,11 @@ edit_get_file_name (const WEdit * edit)
 WEdit *
 find_editor (const WDialog * h)
 {
-    if (edit_widget_is_editor (CONST_WIDGET (h->current->data)))
-        return (WEdit *) h->current->data;
-    return (WEdit *) find_widget_type (h, edit_callback);
+    const WGroup *g = CONST_GROUP (h);
+
+    if (edit_widget_is_editor (CONST_WIDGET (g->current->data)))
+        return (WEdit *) g->current->data;
+    return (WEdit *) widget_find_by_type (CONST_WIDGET (h), edit_callback);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1284,8 +1316,6 @@ edit_widget_is_editor (const Widget * w)
 void
 edit_update_screen (WEdit * e)
 {
-    WDialog *h = WIDGET (e)->owner;
-
     edit_scroll_screen_over_cursor (e);
     edit_update_curs_col (e);
     edit_status (e, widget_get_state (WIDGET (e), WST_FOCUSED));
@@ -1300,7 +1330,7 @@ edit_update_screen (WEdit * e)
         edit_render_keypress (e);
     }
 
-    widget_draw (WIDGET (find_buttonbar (h)));
+    widget_draw (WIDGET (find_buttonbar (DIALOG (WIDGET (e)->owner))));
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1315,10 +1345,7 @@ edit_save_size (WEdit * edit)
 {
     Widget *w = WIDGET (edit);
 
-    edit->y_prev = w->y;
-    edit->x_prev = w->x;
-    edit->lines_prev = w->lines;
-    edit->cols_prev = w->cols;
+    rect_init (&edit->loc_prev, w->y, w->x, w->lines, w->cols);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1350,9 +1377,9 @@ edit_add_window (WDialog * h, int y, int x, int lines, int cols, const vfs_path_
     w->callback = edit_callback;
     w->mouse_callback = edit_mouse_callback;
 
-    add_widget_autopos (h, w, WPOS_KEEP_ALL, NULL);
+    group_add_widget_autopos (GROUP (h), w, WPOS_KEEP_ALL, NULL);
     edit_set_buttonbar (edit, find_buttonbar (h));
-    dlg_draw (h);
+    widget_draw (WIDGET (h));
 
     return TRUE;
 }
