@@ -155,7 +155,7 @@ statfs (char const *filename, struct fs_info *buf)
 
 #include "src/setup.h"          /* verbose, safe_overwrite */
 
-#include "midnight.h"
+#include "filemanager.h"
 #include "fileopctx.h"          /* FILE_CONT */
 
 #include "filegui.h"
@@ -1158,9 +1158,9 @@ file_progress_show_target (file_op_context_t * ctx, const vfs_path_t * vpath)
 gboolean
 file_progress_show_deleting (file_op_context_t * ctx, const char *s, size_t * count)
 {
-    static guint64 timestamp = 0;
+    static gint64 timestamp = 0;
     /* update with 25 FPS rate */
-    static const guint64 delay = G_USEC_PER_SEC / 25;
+    static const gint64 delay = G_USEC_PER_SEC / 25;
 
     gboolean ret;
 
