@@ -1,7 +1,7 @@
 /*
    Directory hotlist -- for the Midnight Commander
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -59,7 +59,6 @@
 #include "src/setup.h"          /* For profile_bname */
 #include "src/history.h"
 
-#include "midnight.h"           /* current_panel */
 #include "command.h"            /* cmdline */
 
 #include "hotlist.h"
@@ -151,6 +150,8 @@ struct hotlist
 
 /*** file scope variables ************************************************************************/
 
+static WPanel *our_panel;
+
 static gboolean hotlist_has_dot_dot = TRUE;
 
 static WDialog *hotlist_dlg, *movelist_dlg;
@@ -216,7 +217,7 @@ static int list_level = 0;
 
 static void init_movelist (struct hotlist *item);
 static void add_new_group_cmd (void);
-static void add_new_entry_cmd (void);
+static void add_new_entry_cmd (WPanel * panel);
 static void remove_from_hotlist (struct hotlist *entry);
 static void load_hotlist (void);
 static void add_dotdot_to_list (void);
@@ -287,6 +288,7 @@ fill_listbox (WListbox * list)
         case HL_TYPE_DOTDOT:
         case HL_TYPE_ENTRY:
             listbox_add_item (list, LISTBOX_APPEND_AT_END, 0, current->label, current, FALSE);
+            break;
         default:
             break;
         }
@@ -413,11 +415,11 @@ hotlist_run_cmd (int action)
         return 0;
 
     case B_ADD_CURRENT:
-        add2hotlist_cmd ();
+        add2hotlist_cmd (our_panel);
         return 0;
 
     case B_NEW_ENTRY:
-        add_new_entry_cmd ();
+        add_new_entry_cmd (our_panel);
         return 0;
 
     case B_ENTER:
@@ -444,7 +446,6 @@ hotlist_run_cmd (int action)
                 fill_listbox (list);
                 return 0;
             }
-            MC_FALLTHROUGH;     /* go up */
         }
         MC_FALLTHROUGH;         /* if list empty - just go up */
 
@@ -607,9 +608,13 @@ hotlist_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
         return MSG_HANDLED;
 
     case MSG_RESIZE:
-        /* simply call dlg_set_size() with new size */
-        dlg_set_size (h, LINES - (h == hotlist_dlg ? 2 : 6), COLS - 6);
-        return MSG_HANDLED;
+        {
+            WRect r;
+
+            rect_init (&r, w->y, w->x, LINES - (h == hotlist_dlg ? 2 : 6), COLS - 6);
+
+            return dlg_default_callback (w, NULL, MSG_RESIZE, 0, &r);
+        }
 
     default:
         return dlg_default_callback (w, sender, msg, parm, data);
@@ -621,7 +626,7 @@ hotlist_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
 static lcback_ret_t
 hotlist_listbox_callback (WListbox * list)
 {
-    WDialog *dlg = WIDGET (list)->owner;
+    WDialog *dlg = DIALOG (WIDGET (list)->owner);
 
     if (!listbox_is_empty (list))
     {
@@ -750,6 +755,7 @@ init_hotlist (hotlist_t list_type)
     int lines, cols;
     int y;
     int dh = 0;
+    WGroup *g;
     WGroupbox *path_box;
     Widget *hotlist_widget;
 
@@ -775,11 +781,12 @@ init_hotlist (hotlist_t list_type)
     hotlist_dlg =
         dlg_create (TRUE, 0, 0, lines, cols, WPOS_CENTER, FALSE, dialog_colors, hotlist_callback,
                     NULL, help_node, title);
+    g = GROUP (hotlist_dlg);
 
     y = UY;
     hotlist_group = groupbox_new (y, UX, lines - 10 + dh, cols - 2 * UX, _("Top level group"));
     hotlist_widget = WIDGET (hotlist_group);
-    add_widget_autopos (hotlist_dlg, hotlist_widget, WPOS_KEEP_ALL, NULL);
+    group_add_widget_autopos (g, hotlist_widget, WPOS_KEEP_ALL, NULL);
 
     l_hotlist =
         listbox_new (y + 1, UX + 1, hotlist_widget->lines - 2, hotlist_widget->cols - 2, FALSE,
@@ -798,26 +805,26 @@ init_hotlist (hotlist_t list_type)
         fill_listbox (l_hotlist);
 
     /* insert before groupbox to view scrollbar */
-    add_widget_autopos (hotlist_dlg, l_hotlist, WPOS_KEEP_ALL, NULL);
+    group_add_widget_autopos (g, l_hotlist, WPOS_KEEP_ALL, NULL);
 
     y += hotlist_widget->lines;
 
     path_box = groupbox_new (y, UX, 3, hotlist_widget->cols, _("Directory path"));
-    add_widget_autopos (hotlist_dlg, path_box, WPOS_KEEP_BOTTOM | WPOS_KEEP_HORZ, NULL);
+    group_add_widget_autopos (g, path_box, WPOS_KEEP_BOTTOM | WPOS_KEEP_HORZ, NULL);
 
     pname = label_new (y + 1, UX + 2, "");
-    add_widget_autopos (hotlist_dlg, pname, WPOS_KEEP_BOTTOM | WPOS_KEEP_LEFT, NULL);
+    group_add_widget_autopos (g, pname, WPOS_KEEP_BOTTOM | WPOS_KEEP_LEFT, NULL);
     y += WIDGET (path_box)->lines;
 
-    add_widget_autopos (hotlist_dlg, hline_new (y++, -1, -1), WPOS_KEEP_BOTTOM, NULL);
+    group_add_widget_autopos (g, hline_new (y++, -1, -1), WPOS_KEEP_BOTTOM, NULL);
 
     for (i = 0; i < hotlist_but_num; i++)
         if ((hotlist_but[i].type & list_type) != 0)
-            add_widget_autopos (hotlist_dlg,
-                                button_new (y + hotlist_but[i].y, UX + hotlist_but[i].x,
-                                            hotlist_but[i].ret_cmd, hotlist_but[i].flags,
-                                            hotlist_but[i].text, hotlist_button_callback),
-                                hotlist_but[i].pos_flags, NULL);
+            group_add_widget_autopos (g,
+                                      button_new (y + hotlist_but[i].y, UX + hotlist_but[i].x,
+                                                  hotlist_but[i].ret_cmd, hotlist_but[i].flags,
+                                                  hotlist_but[i].text, hotlist_button_callback),
+                                      hotlist_but[i].pos_flags, NULL);
 
     widget_select (WIDGET (l_hotlist));
 }
@@ -831,6 +838,7 @@ init_movelist (struct hotlist *item)
     char *hdr;
     int lines, cols;
     int y;
+    WGroup *g;
     Widget *movelist_widget;
 
     do_refresh ();
@@ -843,32 +851,33 @@ init_movelist (struct hotlist *item)
     movelist_dlg =
         dlg_create (TRUE, 0, 0, lines, cols, WPOS_CENTER, FALSE, dialog_colors, hotlist_callback,
                     NULL, "[Hotlist]", hdr);
+    g = GROUP (movelist_dlg);
 
     g_free (hdr);
 
     y = UY;
     movelist_group = groupbox_new (y, UX, lines - 7, cols - 2 * UX, _("Directory label"));
     movelist_widget = WIDGET (movelist_group);
-    add_widget_autopos (movelist_dlg, movelist_widget, WPOS_KEEP_ALL, NULL);
+    group_add_widget_autopos (g, movelist_widget, WPOS_KEEP_ALL, NULL);
 
     l_movelist =
         listbox_new (y + 1, UX + 1, movelist_widget->lines - 2, movelist_widget->cols - 2, FALSE,
                      hotlist_listbox_callback);
     fill_listbox (l_movelist);
     /* insert before groupbox to view scrollbar */
-    add_widget_autopos (movelist_dlg, l_movelist, WPOS_KEEP_ALL, NULL);
+    group_add_widget_autopos (g, l_movelist, WPOS_KEEP_ALL, NULL);
 
     y += movelist_widget->lines;
 
-    add_widget_autopos (movelist_dlg, hline_new (y++, -1, -1), WPOS_KEEP_BOTTOM, NULL);
+    group_add_widget_autopos (g, hline_new (y++, -1, -1), WPOS_KEEP_BOTTOM, NULL);
 
     for (i = 0; i < hotlist_but_num; i++)
         if ((hotlist_but[i].type & LIST_MOVELIST) != 0)
-            add_widget_autopos (movelist_dlg,
-                                button_new (y + hotlist_but[i].y, UX + hotlist_but[i].x,
-                                            hotlist_but[i].ret_cmd, hotlist_but[i].flags,
-                                            hotlist_but[i].text, hotlist_button_callback),
-                                hotlist_but[i].pos_flags, NULL);
+            group_add_widget_autopos (g,
+                                      button_new (y + hotlist_but[i].y, UX + hotlist_but[i].x,
+                                                  hotlist_but[i].ret_cmd, hotlist_but[i].flags,
+                                                  hotlist_but[i].text, hotlist_button_callback),
+                                      hotlist_but[i].pos_flags, NULL);
 
     widget_select (WIDGET (l_movelist));
 }
@@ -1024,13 +1033,13 @@ add_new_entry_input (const char *header, const char *text1, const char *text2,
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-add_new_entry_cmd (void)
+add_new_entry_cmd (WPanel * panel)
 {
     char *title, *url, *to_free;
     int ret;
 
     /* Take current directory as default value for input fields */
-    to_free = title = url = vfs_path_to_str_flags (current_panel->cwd_vpath, 0, VPF_STRIP_PASSWORD);
+    to_free = title = url = vfs_path_to_str_flags (panel->cwd_vpath, 0, VPF_STRIP_PASSWORD);
 
     ret = add_new_entry_input (_("New hotlist entry"), _("Directory label:"),
                                _("Directory path:"), "[Hotlist]", &title, &url);
@@ -1586,7 +1595,7 @@ add_dotdot_to_list (void)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-add2hotlist_cmd (void)
+add2hotlist_cmd (WPanel * panel)
 {
     char *lc_prompt;
     const char *cp = N_("Label for \"%s\":");
@@ -1597,8 +1606,11 @@ add2hotlist_cmd (void)
     cp = _(cp);
 #endif
 
+    /* extra variable to use it in the button callback */
+    our_panel = panel;
+
     l = str_term_width1 (cp);
-    label_string = vfs_path_to_str_flags (current_panel->cwd_vpath, 0, VPF_STRIP_PASSWORD);
+    label_string = vfs_path_to_str_flags (panel->cwd_vpath, 0, VPF_STRIP_PASSWORD);
     lc_prompt = g_strdup_printf (cp, str_trunc (label_string, COLS - 2 * UX - (l + 8)));
     label =
         input_dialog (_("Add to hotlist"), lc_prompt, MC_HISTORY_HOTLIST_ADD, label_string,
@@ -1620,11 +1632,14 @@ add2hotlist_cmd (void)
 /* --------------------------------------------------------------------------------------------- */
 
 char *
-hotlist_show (hotlist_t list_type)
+hotlist_show (hotlist_t list_type, WPanel * panel)
 {
     char *target = NULL;
     int res;
 
+    /* extra variable to use it in the button callback */
+    our_panel = panel;
+
     hotlist_state.type = list_type;
     load_hotlist ();
 
