@@ -50,11 +50,11 @@
 
 #include "src/setup.h"          /* panels_options */
 
-#include "midnight.h"           /* the_menubar */
+#include "filemanager.h"        /* the_menubar */
 #include "layout.h"
 #include "mountlist.h"
 #ifdef ENABLE_EXT2FS_ATTR
-#include "chattr.h"
+#include "cmd.h"                /* chattr_get_as_str() */
 #endif
 
 #include "info.h"
