@@ -60,7 +60,7 @@
 #endif
 
 #include "command.h"
-#include "midnight.h"
+#include "filemanager.h"
 #include "tree.h"
 /* Needed for the extern declarations of integer parameters */
 #include "dir.h"
@@ -1027,9 +1027,9 @@ set_hintbar (const char *str)
 void
 rotate_dash (gboolean show)
 {
-    static guint64 timestamp = 0;
+    static gint64 timestamp = 0;
     /* update with 10 FPS rate */
-    static const guint64 delay = G_USEC_PER_SEC / 10;
+    static const gint64 delay = G_USEC_PER_SEC / 10;
 
     const Widget *w = CONST_WIDGET (midnight_dlg);
 
@@ -1193,7 +1193,7 @@ create_panel (int num, panel_view_mode_t type)
 
     /* We use replace to keep the circular list of the dialog in the */
     /* same state.  Maybe we could just kill it and then replace it  */
-    if ((midnight_dlg != NULL) && (old_widget != NULL))
+    if (old_widget != NULL)
     {
         if (old_type == view_listing)
         {
@@ -1277,8 +1277,8 @@ swap_panels (void)
         panelswap (dir_stat);
 #undef panelswap
 
-        panel1->searching = FALSE;
-        panel2->searching = FALSE;
+        panel1->quick_search.active = FALSE;
+        panel2->quick_search.active = FALSE;
 
         if (current_panel == panel1)
             current_panel = panel2;
@@ -1309,18 +1309,18 @@ swap_panels (void)
 
         if (panels[0].type == view_listing)
         {
-            if (strcmp (panel1->panel_name, get_nth_panel_name (0)) == 0)
+            if (strcmp (panel1->name, get_nth_panel_name (0)) == 0)
             {
-                g_free (panel1->panel_name);
-                panel1->panel_name = g_strdup (get_nth_panel_name (1));
+                g_free (panel1->name);
+                panel1->name = g_strdup (get_nth_panel_name (1));
             }
         }
         if (panels[1].type == view_listing)
         {
-            if (strcmp (panel2->panel_name, get_nth_panel_name (1)) == 0)
+            if (strcmp (panel2->name, get_nth_panel_name (1)) == 0)
             {
-                g_free (panel2->panel_name);
-                panel2->panel_name = g_strdup (get_nth_panel_name (0));
+                g_free (panel2->name);
+                panel2->name = g_strdup (get_nth_panel_name (0));
             }
         }
 
@@ -1498,7 +1498,11 @@ load_prompt (int fd, void *unused)
     (void) fd;
     (void) unused;
 
-    do_load_prompt ();
+    if (should_read_new_subshell_prompt)
+        do_load_prompt ();
+    else
+        flush_subshell (0, QUIETLY);
+
     return 0;
 }
 #endif /* ENABLE_SUBSHELL */
