@@ -6,7 +6,7 @@
    created and destroyed.  This is required for the future vfs layer,
    it will be possible to have tree views over virtual file systems.
 
-   Copyright (C) 1994-2019
+   Copyright (C) 1994-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -60,7 +60,7 @@
 #include "src/history.h"
 
 #include "dir.h"
-#include "midnight.h"           /* the_menubar */
+#include "filemanager.h"        /* the_menubar */
 #include "file.h"               /* copy_dir_dir(), move_dir_dir(), erase_dir() */
 #include "layout.h"             /* command_prompt */
 #include "treestore.h"
@@ -85,10 +85,6 @@ gboolean xtree_mode = FALSE;
 #define tlines(t) (t->is_panel ? WIDGET (t)->lines - 2 - \
                     (panels_options.show_mini_info ? 2 : 0) : WIDGET (t)->lines)
 
-/* Use the color of the parent widget for the unselected entries */
-#define TREE_NORMALC(h) (h->color[DLG_COLOR_NORMAL])
-#define TREE_CURRENTC(h) (h->color[DLG_COLOR_FOCUS])
-
 /*** file scope type declarations ****************************************************************/
 
 struct WTree
@@ -96,7 +92,7 @@ struct WTree
     Widget widget;
     struct TreeStore *store;
     tree_entry *selected_ptr;   /* The selected directory */
-    char search_buffer[MC_MAXFILENAMELEN];      /* Current search string */
+    GString *search_buffer;     /* Current search string */
     tree_entry **tree_shown;    /* Entries currently on screen */
     gboolean is_panel;          /* panel or plain widget flag */
     gboolean searching;         /* Are we on searching mode? */
@@ -199,6 +195,7 @@ tree_destroy (WTree * tree)
     save_tree (tree);
 
     MC_PTR_FREE (tree->tree_shown);
+    g_string_free (tree->search_buffer, TRUE);
     tree->selected_ptr = NULL;
 }
 
@@ -243,15 +240,17 @@ tree_show_mini_info (WTree * tree, int tree_lines, int tree_cols)
         tty_draw_hline (w->y + line, w->x + 1, ' ', tree_cols);
         widget_gotoyx (w, line, 1);
         tty_print_char (PATH_SEP);
-        tty_print_string (str_fit_to_term (tree->search_buffer, tree_cols - 2, J_LEFT_FIT));
+        tty_print_string (str_fit_to_term (tree->search_buffer->str, tree_cols - 2, J_LEFT_FIT));
         tty_print_char (' ');
     }
     else
     {
         /* Show full name of selected directory */
-        WDialog *h = w->owner;
 
-        tty_setcolor (tree->is_panel ? NORMAL_COLOR : TREE_NORMALC (h));
+        const int *colors;
+
+        colors = widget_get_colors (w);
+        tty_setcolor (tree->is_panel ? NORMAL_COLOR : colors[DLG_COLOR_NORMAL]);
         tty_draw_hline (w->y + line, w->x + 1, ' ', tree_cols);
         widget_gotoyx (w, line, 1);
         tty_print_string (str_fit_to_term
@@ -265,7 +264,6 @@ static void
 show_tree (WTree * tree)
 {
     Widget *w = WIDGET (tree);
-    WDialog *h = w->owner;
     tree_entry *current;
     int i, j;
     int topsublevel = 0;
@@ -335,7 +333,10 @@ show_tree (WTree * tree)
     /* Loop for every line */
     for (i = 0; i < tree_lines; i++)
     {
-        tty_setcolor (tree->is_panel ? NORMAL_COLOR : TREE_NORMALC (h));
+        const int *colors;
+
+        colors = widget_get_colors (w);
+        tty_setcolor (tree->is_panel ? NORMAL_COLOR : colors[DLG_COLOR_NORMAL]);
 
         /* Move to the beginning of the line */
         tty_draw_hline (w->y + y + i, w->x + x, ' ', tree_cols);
@@ -344,10 +345,18 @@ show_tree (WTree * tree)
             continue;
 
         if (tree->is_panel)
-            tty_setcolor (widget_get_state (w, WST_FOCUSED) && current == tree->selected_ptr
-                          ? SELECTED_COLOR : NORMAL_COLOR);
+        {
+            gboolean selected;
+
+            selected = widget_get_state (w, WST_FOCUSED) && current == tree->selected_ptr;
+            tty_setcolor (selected ? SELECTED_COLOR : NORMAL_COLOR);
+        }
         else
-            tty_setcolor (current == tree->selected_ptr ? TREE_CURRENTC (h) : TREE_NORMALC (h));
+        {
+            int idx = current == tree->selected_ptr ? DLG_COLOR_FOCUS : DLG_COLOR_NORMAL;
+
+            tty_setcolor (colors[idx]);
+        }
 
         tree->tree_shown[i] = current;
         if (current->sublevel == topsublevel)
@@ -577,21 +586,23 @@ tree_chdir_sel (WTree * tree)
 {
     if (tree->is_panel)
     {
-        change_panel ();
+        WPanel *p;
 
-        if (do_cd (tree->selected_ptr->name, cd_exact))
-            select_item (current_panel);
+        p = change_panel ();
+
+        if (panel_cd (p, tree->selected_ptr->name, cd_exact))
+            select_item (p);
         else
             message (D_ERROR, MSG_ERROR, _("Cannot chdir to \"%s\"\n%s"),
                      vfs_path_as_str (tree->selected_ptr->name), unix_error_string (errno));
 
-        widget_draw (WIDGET (current_panel));
-        change_panel ();
+        widget_draw (WIDGET (p));
+        (void) change_panel ();
         show_tree (tree);
     }
     else
     {
-        WDialog *h = WIDGET (tree)->owner;
+        WDialog *h = DIALOG (WIDGET (tree)->owner);
 
         h->ret_value = B_ENTER;
         dlg_stop (h);
@@ -610,19 +621,15 @@ maybe_chdir (WTree * tree)
 /* --------------------------------------------------------------------------------------------- */
 /** Search tree for text */
 
-static int
-search_tree (WTree * tree, char *text)
+static gboolean
+search_tree (WTree * tree, const GString * text)
 {
-    tree_entry *current;
-    size_t len;
+    tree_entry *current = tree->selected_ptr;
     gboolean wrapped = FALSE;
     gboolean found = FALSE;
 
-    len = strlen (text);
-    current = tree->selected_ptr;
-
     while (!found && (!wrapped || current != tree->selected_ptr))
-        if (strncmp (current->subname, text, len) == 0)
+        if (strncmp (current->subname, text->str, text->len) == 0)
         {
             tree->selected_ptr = current;
             found = TRUE;
@@ -648,19 +655,15 @@ search_tree (WTree * tree, char *text)
 static void
 tree_do_search (WTree * tree, int key)
 {
-    size_t l;
+    /* TODO: support multi-byte characters, see do_search() in panel.c */
 
-    l = strlen (tree->search_buffer);
-    if ((l != 0) && (key == KEY_BACKSPACE))
-        tree->search_buffer[--l] = '\0';
-    else if (key && l < sizeof (tree->search_buffer) - 1)
-    {
-        tree->search_buffer[l] = key;
-        tree->search_buffer[++l] = '\0';
-    }
+    if (tree->search_buffer->len != 0 && key == KEY_BACKSPACE)
+        g_string_set_size (tree->search_buffer, tree->search_buffer->len - 1);
+    else if (key != 0)
+        g_string_append_c (tree->search_buffer, (gchar) key);
 
     if (!search_tree (tree, tree->search_buffer))
-        tree->search_buffer[--l] = '\0';
+        g_string_set_size (tree->search_buffer, tree->search_buffer->len - 1);
 
     show_tree (tree);
     maybe_chdir (tree);
@@ -962,7 +965,7 @@ tree_start_search (WTree * tree)
     else
     {
         tree->searching = TRUE;
-        tree->search_buffer[0] = '\0';
+        g_string_set_size (tree->search_buffer, 0);
     }
 }
 
@@ -971,14 +974,15 @@ tree_start_search (WTree * tree)
 static void
 tree_toggle_navig (WTree * tree)
 {
+    Widget *w = WIDGET (tree);
     WButtonBar *b;
 
     tree_navigation_flag = !tree_navigation_flag;
 
-    b = find_buttonbar (WIDGET (tree)->owner);
+    b = find_buttonbar (DIALOG (w->owner));
     buttonbar_set_label (b, 4,
                          tree_navigation_flag ? Q_ ("ButtonBar|Static") : Q_ ("ButtonBar|Dynamc"),
-                         tree_map, WIDGET (tree));
+                         w->keymap, w);
     widget_draw (WIDGET (b));
 }
 
@@ -1044,7 +1048,7 @@ tree_execute_cmd (WTree * tree, long command)
         break;
     case CK_Quit:
         if (!tree->is_panel)
-            dlg_stop (WIDGET (tree)->owner);
+            dlg_stop (DIALOG (WIDGET (tree)->owner));
         return res;
     default:
         res = MSG_NOT_HANDLED;
@@ -1060,7 +1064,7 @@ tree_execute_cmd (WTree * tree, long command)
 static cb_ret_t
 tree_key (WTree * tree, int key)
 {
-    size_t i;
+    long command;
 
     if (is_abort_char (key))
     {
@@ -1082,18 +1086,19 @@ tree_key (WTree * tree, int key)
         return MSG_HANDLED;
     }
 
-    for (i = 0; tree_map[i].key != 0; i++)
-        if (key == tree_map[i].key)
-            switch (tree_map[i].command)
-            {
-            case CK_Left:
-                return tree_move_left (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
-            case CK_Right:
-                return tree_move_right (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
-            default:
-                tree_execute_cmd (tree, tree_map[i].command);
-                return MSG_HANDLED;
-            }
+    command = widget_lookup_key (WIDGET (tree), key);
+    switch (command)
+    {
+    case CK_IgnoreKey:
+        break;
+    case CK_Left:
+        return tree_move_left (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
+    case CK_Right:
+        return tree_move_right (tree) ? MSG_HANDLED : MSG_NOT_HANDLED;
+    default:
+        tree_execute_cmd (tree, command);
+        return MSG_HANDLED;
+    }
 
     /* Do not eat characters not meant for the tree below ' ' (e.g. C-l). */
     if (!command_prompt && ((key >= ' ' && key <= 255) || key == KEY_BACKSPACE))
@@ -1147,7 +1152,7 @@ static cb_ret_t
 tree_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
     WTree *tree = (WTree *) w;
-    WDialog *h = w->owner;
+    WDialog *h = DIALOG (w->owner);
     WButtonBar *b;
 
     switch (msg)
@@ -1164,20 +1169,20 @@ tree_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *da
 
     case MSG_FOCUS:
         b = find_buttonbar (h);
-        buttonbar_set_label (b, 1, Q_ ("ButtonBar|Help"), tree_map, w);
-        buttonbar_set_label (b, 2, Q_ ("ButtonBar|Rescan"), tree_map, w);
-        buttonbar_set_label (b, 3, Q_ ("ButtonBar|Forget"), tree_map, w);
+        buttonbar_set_label (b, 1, Q_ ("ButtonBar|Help"), w->keymap, w);
+        buttonbar_set_label (b, 2, Q_ ("ButtonBar|Rescan"), w->keymap, w);
+        buttonbar_set_label (b, 3, Q_ ("ButtonBar|Forget"), w->keymap, w);
         buttonbar_set_label (b, 4, tree_navigation_flag ? Q_ ("ButtonBar|Static")
-                             : Q_ ("ButtonBar|Dynamc"), tree_map, w);
-        buttonbar_set_label (b, 5, Q_ ("ButtonBar|Copy"), tree_map, w);
-        buttonbar_set_label (b, 6, Q_ ("ButtonBar|RenMov"), tree_map, w);
+                             : Q_ ("ButtonBar|Dynamc"), w->keymap, w);
+        buttonbar_set_label (b, 5, Q_ ("ButtonBar|Copy"), w->keymap, w);
+        buttonbar_set_label (b, 6, Q_ ("ButtonBar|RenMov"), w->keymap, w);
 #if 0
         /* FIXME: mkdir is currently defunct */
-        buttonbar_set_label (b, 7, Q_ ("ButtonBar|Mkdir"), tree_map, w);
+        buttonbar_set_label (b, 7, Q_ ("ButtonBar|Mkdir"), w->keymap, w);
 #else
         buttonbar_clear_label (b, 7, w);
 #endif
-        buttonbar_set_label (b, 8, Q_ ("ButtonBar|Rmdir"), tree_map, w);
+        buttonbar_set_label (b, 8, Q_ ("ButtonBar|Rmdir"), w->keymap, w);
 
         return MSG_HANDLED;
 
@@ -1226,7 +1231,7 @@ tree_mouse_callback (Widget * w, mouse_msg_t msg, mouse_event_t * event)
             event->result.abort = TRUE;
         }
         else if (!widget_get_state (w, WST_FOCUSED))
-            change_panel ();
+            (void) change_panel ();
         break;
 
     case MSG_MOUSE_CLICK:
@@ -1279,17 +1284,19 @@ tree_new (int y, int x, int lines, int cols, gboolean is_panel)
     Widget *w;
 
     tree = g_new (WTree, 1);
-    w = WIDGET (tree);
 
+    w = WIDGET (tree);
     widget_init (w, y, x, lines, cols, tree_callback, tree_mouse_callback);
     w->options |= WOP_SELECTABLE | WOP_TOP_SELECT;
+    w->keymap = tree_map;
+
     tree->is_panel = is_panel;
     tree->selected_ptr = NULL;
 
     tree->store = tree_store_get ();
     tree_store_add_entry_remove_hook (remove_callback, tree);
     tree->tree_shown = NULL;
-    tree->search_buffer[0] = '\0';
+    tree->search_buffer = g_string_sized_new (MC_MAXPATHLEN);
     tree->topdiff = w->lines / 2;
     tree->searching = FALSE;
 
@@ -1332,9 +1339,9 @@ sync_tree (const vfs_path_t * vpath)
 /* --------------------------------------------------------------------------------------------- */
 
 WTree *
-find_tree (WDialog * h)
+find_tree (const WDialog * h)
 {
-    return (WTree *) find_widget_type (h, tree_callback);
+    return (WTree *) widget_find_by_type (CONST_WIDGET (h), tree_callback);
 }
 
 /* --------------------------------------------------------------------------------------------- */
