@@ -73,7 +73,7 @@ struct mc_fhl_struct;
 
 /*** global variables defined in .c file *********************************************************/
 
-/* global paremeters */
+/* global parameters */
 extern char *global_profile_name;
 extern gboolean confirm_delete;
 extern gboolean confirm_directory_hotlist_delete;
@@ -82,6 +82,7 @@ extern gboolean confirm_exit;
 extern gboolean confirm_overwrite;
 extern gboolean confirm_view_dir;
 extern gboolean safe_delete;
+extern gboolean safe_overwrite;
 extern gboolean clear_before_exec;
 extern gboolean auto_menu;
 extern gboolean drop_menus;
