@@ -1,7 +1,7 @@
 /*
    Virtual File System: External file system.
 
-   Copyright (C) 1995-2019
+   Copyright (C) 1995-2020
    Free Software Foundation, Inc.
 
    Written by:
@@ -200,15 +200,14 @@ extfs_generate_entry (struct extfs_super_t *archive, const char *name, struct vf
     struct vfs_s_inode *inode;
     struct vfs_s_entry *entry;
 
+    memset (&st, 0, sizeof (st));
     st.st_ino = VFS_SUPER (archive)->ino_usage++;
     st.st_dev = archive->rdev;
     myumask = umask (022);
     umask (myumask);
     st.st_mode = mode & ~myumask;
-    st.st_rdev = 0;
     st.st_uid = getuid ();
     st.st_gid = getgid ();
-    st.st_size = 0;
     st.st_mtime = time (NULL);
     st.st_atime = st.st_mtime;
     st.st_ctime = st.st_mtime;
@@ -247,8 +246,6 @@ extfs_find_entry_int (struct vfs_s_inode *dir, const char *name, GSList * list,
 
     while ((pent != NULL) && (c != '\0') && (*p != '\0'))
     {
-        GList *pl;
-
         q = strchr (p, PATH_SEP);
         if (q == NULL)
             q = (char *) name_end;
@@ -256,36 +253,43 @@ extfs_find_entry_int (struct vfs_s_inode *dir, const char *name, GSList * list,
         c = *q;
         *q = '\0';
 
-        pent = extfs_resolve_symlinks_int (pent, list);
-        if (pent == NULL)
+        if (DIR_IS_DOTDOT (p))
+            pent = pent->dir->ent;
+        else
         {
-            *q = c;
-            return NULL;
-        }
+            GList *pl;
 
-        if (!S_ISDIR (pent->ino->st.st_mode))
-        {
-            *q = c;
-            notadir = TRUE;
-            return NULL;
-        }
+            pent = extfs_resolve_symlinks_int (pent, list);
+            if (pent == NULL)
+            {
+                *q = c;
+                return NULL;
+            }
 
-        pdir = pent;
-        pl = g_queue_find_custom (pent->ino->subdir, p, vfs_s_entry_compare);
-        pent = pl != NULL ? VFS_ENTRY (pl->data) : NULL;
-        if (pent != NULL && q + 1 > name_end)
-        {
-            /* Hack: I keep the original semanthic unless q+1 would break in the strchr */
-            *q = c;
-            notadir = !S_ISDIR (pent->ino->st.st_mode);
-            return pent;
-        }
+            if (!S_ISDIR (pent->ino->st.st_mode))
+            {
+                *q = c;
+                notadir = TRUE;
+                return NULL;
+            }
 
-        /* When we load archive, we create automagically non-existent directories */
-        if (pent == NULL && (flags & FL_MKDIR) != 0)
-            pent = extfs_generate_entry (super, p, pdir->ino, S_IFDIR | 0777);
-        if (pent == NULL && (flags & FL_MKFILE) != 0)
-            pent = extfs_generate_entry (super, p, pdir->ino, S_IFREG | 0666);
+            pdir = pent;
+            pl = g_queue_find_custom (pent->ino->subdir, p, vfs_s_entry_compare);
+            pent = pl != NULL ? VFS_ENTRY (pl->data) : NULL;
+            if (pent != NULL && q + 1 > name_end)
+            {
+                /* Hack: I keep the original semanthic unless q+1 would break in the strchr */
+                *q = c;
+                notadir = !S_ISDIR (pent->ino->st.st_mode);
+                return pent;
+            }
+
+            /* When we load archive, we create automagically non-existent directories */
+            if (pent == NULL && (flags & FL_MKDIR) != 0)
+                pent = extfs_generate_entry (super, p, pdir->ino, S_IFDIR | 0777);
+            if (pent == NULL && (flags & FL_MKFILE) != 0)
+                pent = extfs_generate_entry (super, p, pdir->ino, S_IFREG | 0666);
+        }
 
         /* Next iteration */
         *q = c;
@@ -368,6 +372,26 @@ extfs_free_archive (struct vfs_class *me, struct vfs_s_super *psup)
 
 /* --------------------------------------------------------------------------------------------- */
 
+static inline char *
+extfs_skip_leading_dotslash (char *s)
+{
+    /* Skip leading "./" (if present).
+     * Some programs don't understand it:
+     *
+     * $ zip file.zip ./-file2.txt file1.txt
+     *   adding: -file2.txt (stored 0%)
+     *   adding: file1.txt (stored 0%)
+     * $ /usr/lib/mc/extfs.d/uzip copyout file.zip ./-file2.txt ./tmp-file2.txt
+     * caution: filename not matched:  ./-file2.txt
+     */
+    if (s[0] == '.' && s[1] == PATH_SEP)
+        s += 2;
+
+    return s;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+
 static FILE *
 extfs_open_archive (int fstype, const char *name, struct extfs_super_t **pparc)
 {
@@ -383,6 +407,8 @@ extfs_open_archive (int fstype, const char *name, struct extfs_super_t **pparc)
     vfs_path_t *local_name_vpath = NULL;
     vfs_path_t *name_vpath;
 
+    memset (&mystat, 0, sizeof (mystat));
+
     name_vpath = vfs_path_from_str (name);
     info = &g_array_index (extfs_plugins, extfs_plugin_info_t, fstype);
 
@@ -481,6 +507,7 @@ extfs_read_archive (FILE * extfsd, struct extfs_super_t *current_archive)
 
             if (*cfn != '\0')
             {
+                cfn = extfs_skip_leading_dotslash (cfn);
                 if (IS_PATH_SEP (*cfn))
                     cfn++;
                 p = strchr (cfn, '\0');
@@ -537,14 +564,13 @@ extfs_read_archive (FILE * extfsd, struct extfs_super_t *current_archive)
                 {
                     struct stat st;
 
+                    memset (&st, 0, sizeof (st));
                     st.st_ino = super->ino_usage++;
                     st.st_nlink = 1;
                     st.st_dev = current_archive->rdev;
                     st.st_mode = hstat.st_mode;
 #ifdef HAVE_STRUCT_STAT_ST_RDEV
                     st.st_rdev = hstat.st_rdev;
-#else
-                    st.st_rdev = 0;
 #endif
                     st.st_uid = hstat.st_uid;
                     st.st_gid = hstat.st_gid;
@@ -553,11 +579,8 @@ extfs_read_archive (FILE * extfsd, struct extfs_super_t *current_archive)
                     st.st_atime = hstat.st_atime;
                     st.st_ctime = hstat.st_ctime;
 
-                    if (current_link_name == NULL || !S_ISLNK (hstat.st_mode))
-                    {
-                        if (S_ISLNK (hstat.st_mode))
-                            st.st_mode &= ~S_IFLNK;     /* You *DON'T* want to do this always */
-                    }
+                    if (current_link_name == NULL && S_ISLNK (hstat.st_mode))
+                        st.st_mode &= ~S_IFLNK; /* You *DON'T* want to do this always */
 
                     inode = vfs_s_new_inode (super->me, super, &st);
                     inode->ent = entry;
@@ -565,7 +588,7 @@ extfs_read_archive (FILE * extfsd, struct extfs_super_t *current_archive)
 
                     if (current_link_name != NULL && S_ISLNK (hstat.st_mode))
                     {
-                        VFS_INODE (inode)->linkname = current_link_name;
+                        inode->linkname = current_link_name;
                         current_link_name = NULL;
                     }
                 }
@@ -810,13 +833,16 @@ extfs_cmd (const char *str_extfs_cmd, const struct extfs_super_t *archive,
     quoted_file = name_quote (file, FALSE);
     g_free (file);
 
+    /* Skip leading "./" (if present) added in name_quote() */
+    file = extfs_skip_leading_dotslash (quoted_file);
+
     archive_name = extfs_get_archive_name (archive);
     quoted_archive_name = name_quote (archive_name, FALSE);
     g_free (archive_name);
     quoted_localname = name_quote (localname, FALSE);
     info = &g_array_index (extfs_plugins, extfs_plugin_info_t, archive->fstype);
     cmd = g_strconcat (info->path, info->prefix, str_extfs_cmd,
-                       quoted_archive_name, " ", quoted_file, " ", quoted_localname, (char *) NULL);
+                       quoted_archive_name, " ", file, " ", quoted_localname, (char *) NULL);
     g_free (quoted_file);
     g_free (quoted_localname);
     g_free (quoted_archive_name);
@@ -1021,20 +1047,20 @@ extfs_opendir (const vfs_path_t * vpath)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static void *
+static struct vfs_dirent *
 extfs_readdir (void *data)
 {
-    static union vfs_dirent dir;
+    struct vfs_dirent *dir;
     GList **info = (GList **) data;
 
     if (*info == NULL)
         return NULL;
 
-    g_strlcpy (dir.dent.d_name, VFS_ENTRY ((*info)->data)->name, MC_MAXPATHLEN);
+    dir = vfs_dirent_init (NULL, VFS_ENTRY ((*info)->data)->name, 0);   /* FIXME: inode */
 
     *info = g_list_next (*info);
 
-    return (void *) &dir;
+    return dir;
 }
 
 /* --------------------------------------------------------------------------------------------- */
