@@ -2,13 +2,13 @@
    Virtual File System: FISH implementation for transfering files over
    shell connections.
 
-   Copyright (C) 1998-2019
+   Copyright (C) 1998-2020
    Free Software Foundation, Inc.
 
    Written by:
    Pavel Machek, 1998
    Michal Svec, 2000
-   Andrew Borodin <aborodin@vmail.ru>, 2010
+   Andrew Borodin <aborodin@vmail.ru>, 2010-2019
    Slava Zanko <slavazanko@gmail.com>, 2010, 2013
    Ilia Maslakov <il.smind@gmail.com>, 2010
 
@@ -54,7 +54,6 @@
 #include <errno.h>
 #include <pwd.h>
 #include <grp.h>
-#include <sys/time.h>           /* gettimeofday() */
 #include <stdlib.h>
 #include <string.h>
 #include <inttypes.h>           /* uintmax_t */
@@ -489,7 +488,7 @@ fish_info (struct vfs_class *me, struct vfs_s_super *super)
         while (TRUE)
         {
             int res;
-            char buffer[BUF_8K];
+            char buffer[BUF_8K] = "";
 
             res = vfs_s_get_line_interruptible (me, buffer, sizeof (buffer), fish_super->sockr);
             if ((res == 0) || (res == EINTR))
@@ -766,8 +765,7 @@ fish_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path)
 
     vfs_print_message (_("fish: Reading directory %s..."), remote_path);
 
-    gettimeofday (&dir->timestamp, NULL);
-    dir->timestamp.tv_sec += fish_directory_timeout;
+    dir->timestamp = g_get_real_time () + fish_directory_timeout * G_USEC_PER_SEC;
 
     quoted_path = strutils_shell_escape (remote_path);
     (void) fish_command_v (me, super, NONE, FISH_SUPER (super)->scr_ls, "FISH_FILENAME=%s;\n",
@@ -919,6 +917,7 @@ fish_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path)
             {
                 struct tm tim;
 
+                memset (&tim, 0, sizeof (tim));
                 /* cppcheck-suppress invalidscanf */
                 if (sscanf (buffer + 1, "%d %d %d %d %d %d", &tim.tm_year, &tim.tm_mon,
                             &tim.tm_mday, &tim.tm_hour, &tim.tm_min, &tim.tm_sec) != 6)
@@ -940,6 +939,7 @@ fish_dir_load (struct vfs_class *me, struct vfs_s_inode *dir, char *remote_path)
                 ST.st_rdev = makedev (maj, min);
 #endif
             }
+            break;
         default:
             break;
         }
