@@ -21,6 +21,8 @@
 #define LIBSSH2_INVALID_SOCKET -1
 #endif
 
+#define SFTP_SUPER(super) ((sftpfs_super_t *) (super))
+
 /*** enums ***************************************************************************************/
 
 typedef enum
@@ -35,6 +37,8 @@ typedef enum
 
 typedef struct
 {
+    struct vfs_s_super base;
+
     sftpfs_auth_type_t auth_type;
     sftpfs_auth_type_t config_auth_type;
 
@@ -49,27 +53,24 @@ typedef struct
     int socket_handle;
     const char *fingerprint;
     vfs_path_element_t *original_connection_info;
-} sftpfs_super_data_t;
+} sftpfs_super_t;
 
 /*** global variables defined in .c file *********************************************************/
 
 extern GString *sftpfs_filename_buffer;
-extern struct vfs_class sftpfs_class;
 extern struct vfs_s_subclass sftpfs_subclass;
+extern struct vfs_class *sftpfs_class;
 
 /*** declarations of public functions ************************************************************/
 
 void sftpfs_init_class (void);
 void sftpfs_init_subclass (void);
-void sftpfs_init_class_callbacks (void);
-void sftpfs_init_subclass_callbacks (void);
 void sftpfs_init_config_variables_patterns (void);
 void sftpfs_deinit_config_variables_patterns (void);
 
 gboolean sftpfs_is_sftp_error (LIBSSH2_SFTP * sftp_session, int sftp_res, int sftp_error);
-void sftpfs_ssherror_to_gliberror (sftpfs_super_data_t * super_data, int libssh_errno,
-                                   GError ** mcerror);
-gboolean sftpfs_waitsocket (sftpfs_super_data_t * super_data, int sftp_res, GError ** mcerror);
+void sftpfs_ssherror_to_gliberror (sftpfs_super_t * super, int libssh_errno, GError ** mcerror);
+gboolean sftpfs_waitsocket (sftpfs_super_t * super, int sftp_res, GError ** mcerror);
 
 const char *sftpfs_fix_filename (const char *file_name, unsigned int *length);
 void sftpfs_attr_to_stat (const LIBSSH2_SFTP_ATTRIBUTES * attrs, struct stat *s);
@@ -87,21 +88,21 @@ int sftpfs_open_connection (struct vfs_s_super *super, GError ** mcerror);
 void sftpfs_close_connection (struct vfs_s_super *super, const char *shutdown_message,
                               GError ** mcerror);
 
+vfs_file_handler_t *sftpfs_fh_new (struct vfs_s_inode *ino, gboolean changed);
+
 void *sftpfs_opendir (const vfs_path_t * vpath, GError ** mcerror);
-void *sftpfs_readdir (void *data, GError ** mcerror);
+struct vfs_dirent *sftpfs_readdir (void *data, GError ** mcerror);
 int sftpfs_closedir (void *data, GError ** mcerror);
 int sftpfs_mkdir (const vfs_path_t * vpath, mode_t mode, GError ** mcerror);
 int sftpfs_rmdir (const vfs_path_t * vpath, GError ** mcerror);
 
-gboolean sftpfs_open_file (vfs_file_handler_t * file_handler, int flags, mode_t mode,
+gboolean sftpfs_open_file (vfs_file_handler_t * fh, int flags, mode_t mode, GError ** mcerror);
+ssize_t sftpfs_read_file (vfs_file_handler_t * fh, char *buffer, size_t count, GError ** mcerror);
+ssize_t sftpfs_write_file (vfs_file_handler_t * fh, const char *buffer, size_t count,
                            GError ** mcerror);
-ssize_t sftpfs_read_file (vfs_file_handler_t * file_handler, char *buffer, size_t count,
-                          GError ** mcerror);
-ssize_t sftpfs_write_file (vfs_file_handler_t * file_handler, const char *buffer, size_t count,
-                           GError ** mcerror);
-int sftpfs_close_file (vfs_file_handler_t * file_handler, GError ** mcerror);
+int sftpfs_close_file (vfs_file_handler_t * fh, GError ** mcerror);
 int sftpfs_fstat (void *data, struct stat *buf, GError ** mcerror);
-off_t sftpfs_lseek (vfs_file_handler_t * file_handler, off_t offset, int whence, GError ** mcerror);
+off_t sftpfs_lseek (vfs_file_handler_t * fh, off_t offset, int whence, GError ** mcerror);
 
 /*** inline functions ****************************************************************************/
 
