@@ -65,7 +65,6 @@ teardown (void)
 static void
 test_init_vfs (const char *encoding)
 {
-    mc_global.timer = mc_timer_new ();
     str_init_strings (encoding);
 
     vfs_init ();
@@ -86,7 +85,6 @@ test_deinit_vfs (void)
     free_codepages_list ();
     str_uninit_strings ();
     vfs_shut ();
-    mc_timer_destroy (mc_global.timer);
 }
 
 /* --------------------------------------------------------------------------------------------- */
