@@ -76,7 +76,6 @@ mc_stat (const vfs_path_t * vpath, struct stat *my_stat)
 static void
 setup (void)
 {
-    mc_global.timer = mc_timer_new ();
     str_init_strings (NULL);
 
     vfs_init ();
@@ -92,7 +91,6 @@ teardown (void)
 {
     vfs_shut ();
     str_uninit_strings ();
-    mc_timer_destroy (mc_global.timer);
 }
 
 /* --------------------------------------------------------------------------------------------- */
