@@ -95,7 +95,6 @@ mc_config_get_string__deinit (void)
 static void
 setup (void)
 {
-    mc_global.timer = mc_timer_new ();
     str_init_strings (NULL);
     vfs_init ();
     vfs_init_localfs ();
@@ -114,7 +113,6 @@ teardown (void)
 
     vfs_shut ();
     str_uninit_strings ();
-    mc_timer_destroy (mc_global.timer);
 }
 
 /* --------------------------------------------------------------------------------------------- */
